/***************************************************************************
 *   Copyright (C) 2001  Michael Schänzler                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "lcd_dip122.h"
#include <string.h>

static void render (glyph *g);
static void font_write_char (char c);
static void getglyph (char c, glyph *g);
static uint8_t get_bitmap_pixel (uint8_t x, uint8_t y,
				 uint8_t dx, uint8_t dy,
				 uint8_t *pixel_data);
static lcd_font font_desc = {
  0,0,0,0,0,0,0,0,0,0,0,0,0,0};

void lcd_init_font_handler_small (lcd_font *font)
{
  lcd_current_font_handler.char_writer = font_write_char;
  lcd_current_font_handler.glyph_getter = getglyph;
  lcd_current_font_handler.font_data = (uint8_t *)&font_desc;
  memcpy_P (&font_desc, font, sizeof(lcd_font));
  write_char = font_write_char;
}

static void font_write_char (char c)
{
  glyph g;

  if ((uint8_t)c<font_desc.ascii_min)
    return;

  getglyph (c, &g);
  render (&g);

  lcd_cursor_x += g.dwidth_x + font_desc.BSP;
}

static void render (glyph *g)
{
  lcd_drawpixeldata_generic (lcd_cursor_x + g->bbx_xoff, 
			     lcd_cursor_y + g->bbx_yoff - font_desc.descent - 1,
			     g->bbx_width, g->bbx_height,
			     (uint8_t *)(g->pixel),
			     lcd_drawing_mode,
			     get_bitmap_pixel);
}

static void getglyph (char c, glyph *g)
{
  uint16_t toc1;
  uint16_t toc2;
  uint16_t glyph_offset1;
  uint16_t glyph_offset2;
  uint8_t header_byte1;
  uint8_t header_byte2;
  uint8_t *pixel;
  uint8_t nbytes;
  uint8_t width;
  uint8_t dwidth_x;
  int8_t xoff;
  uint8_t yoff;

  if ((uint8_t)c < 128) {
    toc1 = c - font_desc.ascii_min;
    toc2 = (c+1) - font_desc.ascii_min;
  }
  else {
    toc1 = (uint8_t)c - font_desc.ascii_min - 33;
    toc2 = (uint8_t)(c+1) - font_desc.ascii_min - 33;
  }

  memcpy_P (&glyph_offset1, font_desc.toc + toc1, sizeof(glyph_offset1));
  memcpy_P (&glyph_offset2, font_desc.toc + toc2, sizeof(glyph_offset2));

  header_byte1 = pgm_read_byte(font_desc.glyphs + glyph_offset1);
  dwidth_x = (header_byte1 & 0xf0) >> 4;
  width = header_byte1 & 0x0f;

  header_byte2 = pgm_read_byte(font_desc.glyphs + glyph_offset1 + 1);
  xoff = ((header_byte2 & 0xe0) >> 5) - 3;
  yoff = header_byte2 & 0x1f;

  pixel = (uint8_t *)(font_desc.glyphs + glyph_offset1 + 2);
  nbytes = glyph_offset2 - glyph_offset1 - 2;

  g->bbx_height = 8*nbytes/width;
  g->bbx_width = width;
  g->pixel = pixel;
  g->dwidth_x = dwidth_x; /* width + font_desc.BSP; */
  g->bbx_yoff = yoff;
  g->bbx_xoff = xoff;
}

static uint8_t get_bitmap_pixel (uint8_t x, uint8_t y,
				 uint8_t dx,
				 uint8_t dy __attribute__ ((unused)),
				 uint8_t *pixel_data)
{
  uint16_t byte_offset;
  uint8_t bitpos;

  byte_offset = ((y*dx)%8 + x)/8 + (y*dx)/8;

  bitpos = ((y*dx)%8 + x)%8;

  return pgm_read_byte(pixel_data + byte_offset) & (1 << bitpos);
}

